@@ -6,10 +6,11 @@ Created on 13/10/2012
 
 @author: iuri
 '''
-
-from django.db import models, connection
+import sys
+from django.db import models
 from django_orm.postgresql import hstore
 from django.utils.datastructures import SortedDict
+from django.db.models.fields import FieldDoesNotExist
 from django import forms
 
 from datetime import datetime
@@ -19,6 +20,7 @@ import json
 import copy
 
 from fields import *
+from queryset import *
 from utils import *
 
 
@@ -49,7 +51,7 @@ class DynamicField(models.Model):
 
     def save(self, *args, **kwargs):
         super(DynamicField, self).save(*args, **kwargs)
-        # NOTE: force update global dfields resultset
+        # NOTE: force update global dfields fieldset
         global dfields
         dfields =  DynamicField.objects.all()
 
@@ -59,16 +61,11 @@ dfields =  DynamicField.objects.all()
 
 def find_dfields(refer=None, name=None):
     if name and refer:
-        return [dfield for dfield in dfields \
-            if dfield.refer == refer and dfield.name == name]
-    if name and not refer:
-        [dfield for dfield in dfields if dfield.name == name]
-    return [dfield for dfield in dfields if dfield.refer == refer]
-
-# NOTE: Error happen on syncdb, because DynamicField's table does not exist.
-cursor = connection.cursor()
-cursor.execute("select count(*) from pg_tables where tablename='dynamic_field'")
-DYNAMIC_FIELD_TABLE_EXIST = (cursor.fetchone()[0] > 0)
+        return [dfield for dfield in dfields if dfield.refer == refer and dfield.name == name]
+    elif name:
+        return [dfield for dfield in dfields if dfield.name == name]
+    elif refer:
+        return [dfield for dfield in dfields if dfield.refer == refer]
 
 
 class HStoreModelMeta(models.Model.__metaclass__):
@@ -83,6 +80,8 @@ class HStoreModelMeta(models.Model.__metaclass__):
         # override getattr/setattr/delattr
         old_getattribute = new_class.__getattribute__
         def __getattribute__(self, key):
+            field = None
+
             try:
                 return old_getattribute(self, key)
             except AttributeError:
@@ -95,6 +94,16 @@ class HStoreModelMeta(models.Model.__metaclass__):
                     return field[0].default_value
                 else:
                     raise
+            except ValueError:
+                if isinstance(field, list) and field:
+                    return field[0].default_value
+            except TypeError:
+                field = find_dfields(refer=self.__class__.__name__, name=key)
+
+                if field and field.__class__.__name__ == 'ManyRelatedManager':
+                    return field.all()
+
+                return field
 
         new_class.__getattribute__ = __getattribute__
 
@@ -131,65 +140,114 @@ class HStoreModelMeta(models.Model.__metaclass__):
         # override _meta.fields (property)
         _old_meta = new_class._meta
         class _meta(object):
+            def __eq__(self, other):
+                return _old_meta == other
+
+            def __getattr__(self, key):
+                return getattr(_old_meta, key)
+            def __setattr__(self, key, value):
+                return setattr(_old_meta, key, value)
+
+            def init_name_map(self):
+                _cache = _old_meta.init_name_map()
+
+                for dfield in self.dynamic_fields:
+                    _cache.update(**{
+                        dfield.name: (dfield, _old_meta.concrete_model, True, False)
+                    })
+
+                return _cache
+
+            def get_field_by_name(self, name):
+                if name is 'pk': name = 'id'
+
+                try:
+                    if hasattr(self, '_name_map') and name in self._name_map:
+                        return self._name_map[name]
+                    else:
+                        cache = self.init_name_map()
+                        return cache[name]
+                except KeyError:
+                    raise FieldDoesNotExist('%s has no field named %r'
+                            % (self.object_name, name))
+
+            def get_field(self, name, many_to_many=True):
+                """
+                Returns the requested field by name. Raises FieldDoesNotExist on error.
+                """
+                to_search = many_to_many and (self.fields + self.many_to_many) or self.fields
+                for f in to_search:
+                    if f.name == name:
+                        return f
+
+                raise FieldDoesNotExist('%s has no field named %r' % (self.object_name, name))
+
+            def get_all_field_names(self):
+                return [f.name for f in self.fields if not f.name == '_dfields']
+
+            def get_all_dynamic_field_names(self):
+                return [f.name for f in self.dynamic_fields if not f.name == '_dfields']
+
             @property
             def dynamic_fields(self):
                 fields = []
-                if not DYNAMIC_FIELD_TABLE_EXIST:
+
+                if not dynamic_field_table_exists():
                     return fields
 
                 # metafields = DynamicField.objects.filter(refer=new_class.__name__)
                 metafields = find_dfields(refer=new_class.__name__)
                 for metafield in metafields:
                     try:
-                        field_klass_name = FIELD_TYPES_DICT.get(metafield.typo,
-                                                                FIELD_TYPE_DEFAULT)
-
-                        #FIXME: eval is the evil, use module package
-                        field_klass = eval(field_klass_name)
-                        if metafield.choices == '':
-                            choices_ = None
-                        else:
-                            choices_ = single_list_to_tuple([\
-                                s.strip() for s in metafield.choices.splitlines()
-                            ])
-
-                        field = field_klass(name=metafield.name,
-                                            max_length=metafield.max_length or 255,
-                                            choices=choices_,
-                                            default=metafield.default_value,
-                                            verbose_name=metafield.verbose_name,
-                                            blank=metafield.blank,
-                                            null=True)
-                        field.attname = metafield.name
-                        fields.append(field)
-                    except:
+                        fields.append(
+                            crate_field_from_instance(metafield)
+                        )
+                    except SyntaxError:
                         raise \
                             TypeError(('Cannot create field for %r, maybe type %r ' + \
-                                       'is not a django type') % (metafield, field_klass_name))
+                                       'is not a django type') % (metafield, metafield.typo))
                 return fields
 
-            def __eq__(self, other):
-                return _old_meta == other
-
             @property
             def fields(self):
                 #add dynamic_fields from table
                 return _old_meta.fields + self.dynamic_fields
 
-            def __getattr__(self, key):
-                return getattr(_old_meta, key)
-            def __setattr__(self, key, value):
-                return setattr(_old_meta, key, value)
-        new_class._meta = _meta()
+            def get_base_chain(self, model):
+                """
+                Returns a list of parent classes leading to 'model' (order from closet
+                to most distant ancestor). This has to handle the case were 'model' is
+                a granparent or even more distant relation.
+                """
 
-        # return it
+                if model in self.parents or not self.parents:
+                    # FIXME: In cases of the actual Model doesn`t have
+                    # Any parent, so return him
+                    return [model]
+
+                parent = None
+                for parent in self.parents:
+                    res = parent._meta.get_base_chain(model)
+                    if res:
+                        res.insert(0, parent)
+                        return res
+
+                if model.__base__ == parent:
+                    return [parent]
+
+                raise TypeError('%r is not an ancestor of this model'
+                        % model._meta.module_name)
+
+        new_class._meta = _meta()
         return new_class
 
+
 class HStoreModel(models.Model):
     __metaclass__ = HStoreModelMeta
-    objects = hstore.HStoreManager()
     _dfields = hstore.DictionaryField(db_index=True, null=True, blank=True)
 
+    objects = FlattenFieldsFilterManager()
+
     class Meta:
         abstract = True
 
